@@ -136,33 +136,70 @@ sub list : Chained('base') Args(0) {
     my $limit = $c->req->params->{'limit'} || 10;
     my $sort  = $c->req->params->{'sort'}  || $info->{pk};
     (my $dir  = $c->req->params->{'dir'}   || 'ASC') =~ s/\s//g;
-    my $delayed_paging = 0; # in case we filter/sort on FK
+    my $filter = {}; my $search_opts = {};
+
+    # we want to prefetch all related data for _sfy
+    foreach my $rel (keys %{$info->{cols}}) {
+        next unless ($info->{cols}->{$rel}->{is_fk} or $info->{cols}->{$rel}->{is_rr});
+        #my $join_to = $lf->{table_info}->{$info->{cols}->{$rel}->{fk_model}}->{path};
+        push @{$search_opts->{prefetch}}, $rel;
+    }
+
+    # FIXME until subquery support is used, must not prefetch _many relations
+    # FIXME must also add DBIx::Class dependency of 0.08109
+    #foreach my $rel (keys %{$info->{mfks}}) {
+    #    if (exists $info->{m2m}->{$rel}) {
+    #        my $target = $info->{m2m}->{$rel};
+    #        push @{$search_opts->{prefetch}}, { $rel => $target };
+    #    }
+    #    else {
+    #        push @{$search_opts->{prefetch}}, $rel;
+    #    }
+    #}
 
     # sanity check the sort param
     $sort = $info->{pk} if $sort !~ m/^[\w ]+$/ or !exists $info->{cols}->{$sort};
 
-    # set up pager, if needed (if user sorting by FK then delay paging)
-    my $search_opts = (
-        ($page =~ m/^\d+$/ and $limit =~ m/^\d+$/ and ! $info->{cols}->{$sort}->{is_fk})
-        ? { 'page' => $page, 'rows' => $limit, } : {});
-
-    # find filter fields in UI form
-    my $filter = {};
+    # before setting up the paging and sorting, we need to check whether
+    # the FK params are legit PK vals in the related schema
+    my %delay_page_sort = ();
     foreach my $p (keys %{$c->req->params}) {
-        next unless $p =~ m/^search\.([\w ]+)/;
-        my $col = $1;
+        next unless (my $col) = ($p =~ m/^search\.([\w ]+)/);
+        next unless exists $info->{cols}->{$col}
+            and ($info->{cols}->{$col}->{is_fk} or $info->{cols}->{$col}->{is_rr});
+        my $rs = $c->model($lf->{model})
+                    ->result_source->related_source($col)->resultset;
+        # cannot page or sort this col in the DB if it's not a legit PK val
+        $delay_page_sort{$col} += 1
+            if !defined $rs->find( $c->req->params->{"search.$col"} );
+    }
+
+    # find filter fields in UI form that can be passed to DB
+    foreach my $p (keys %{$c->req->params}) {
+        next unless (my $col) = ($p =~ m/^search\.([\w ]+)/);
         next unless exists $info->{cols}->{$col};
-        if ($info->{cols}->{$col}->{is_fk} or $info->{cols}->{$col}->{is_rr}) {
-            # sorry, have to kill the paging here because it breaks filtering on FK
-            $search_opts = {};
-            $delayed_paging = 1;
+        next if exists $delay_page_sort{$col};
+
+        # search for exact match on FK value (checked above)
+        if ($info->{cols}->{$col}->{is_fk}) {
+            my $masked_col = (exists $info->{cols}->{$col}->{masked_col}
+                ? $info->{cols}->{$col}->{masked_col} : $col);
+            $filter->{"me.$masked_col"} = $c->req->params->{"search.$col"};
+            next;
+        }
+
+        if ($info->{cols}->{$col}->{is_rr}) {
+            next if !exists $info->{cols}->{$col}->{foreign_col};
+            my $foreign_col = $info->{cols}->{$col}->{foreign_col};
+            push @{$search_opts->{join}}, $col;
+            $filter->{"$col.$foreign_col"} = $c->req->params->{"search.$col"};
             next;
         }
 
         # for numberish types the case insensitive functions may not work
         if (exists $info->{cols}->{$col}->{extjs_xtype}
             and $info->{cols}->{$col}->{extjs_xtype} eq 'numberfield') {
-            $filter->{$col} = $c->req->params->{"search.$col"};
+            $filter->{"me.$col"} = $c->req->params->{"search.$col"};
             next;
         }
 
@@ -174,15 +211,27 @@ sub list : Chained('base') Args(0) {
         };
     }
 
+    # any sort on FK -must- disable DB-side paging, unless we already know the
+    # supplied filter is a legitimate PK of the related table
+    if (($info->{cols}->{$sort}->{is_fk} or $info->{cols}->{$sort}->{is_rr})
+            and not (exists $c->req->params->{"search.$sort"} and not exists $delay_page_sort{$sort})) {
+        $delay_page_sort{$sort} += 1;
+    }
+
     # sort col which can be passed to the db
-    if ($dir =~ m/^(?:ASC|DESC)$/ and ! $info->{cols}->{$sort}->{is_fk}
-                                  and ! $info->{cols}->{$sort}->{is_rr}) {
+    if ($dir =~ m/^(?:ASC|DESC)$/ and !exists $delay_page_sort{$sort}
+        and not ($info->{cols}->{$sort}->{is_fk} or $info->{cols}->{$sort}->{is_rr})) {
         $search_opts->{order_by} = \"me.$sort $dir";
     }
 
+    # set up pager, if needed (if user filtering by FK then delay paging)
+    if ($page =~ m/^\d+$/ and $limit =~ m/^\d+$/ and not scalar keys %delay_page_sort) {
+        $search_opts->{page} = $page;
+        $search_opts->{rows} = $limit;
+    }
+
     my $rs = $c->model($lf->{model})->search($filter, $search_opts);
-    my @columns = (exists $search_opts->{columns} ? @{$search_opts->{columns}}
-                                                  : keys %{ $info->{cols} });
+    my @columns = keys %{ $info->{cols} };
 
     #$c->model($lf->{model})->result_source->storage->debug(1)
     #    if $c->debug;
@@ -224,13 +273,12 @@ sub list : Chained('base') Args(0) {
         push @{$response->{rows}}, $data;
     }
 
-    # apply any filters on FK
-    foreach my $p (keys %{$c->req->params}) {
-        next unless $p =~ m/^search\.([\w ]+)/;
-        my $col = $1;
-        next unless exists $info->{cols}->{$col};
-        next unless ($info->{cols}->{$col}->{is_fk} or $info->{cols}->{$col}->{is_rr});
+    #$c->model($lf->{model})->result_source->storage->debug(0)
+    #    if $c->debug;
 
+    # apply any filters on FK
+    foreach my $col (keys %delay_page_sort) {
+        next unless exists $c->req->params->{"search.$col"};
         my $p_val = $c->req->params->{"search.$col"};
         my $fk_match = ($p_val ? qr/\Q$p_val\E/i : qr/./);
 
@@ -241,7 +289,7 @@ sub list : Chained('base') Args(0) {
     }
 
     # sort col which cannot be passed to the DB
-    if ($info->{cols}->{$sort}->{is_fk} or $info->{cols}->{$sort}->{is_rr}) {
+    if (exists $delay_page_sort{$sort}) {
         @{$response->{rows}} = sort {
             $dir eq 'ASC' ? ($a->{$sort} cmp $b->{$sort})
                           : ($b->{$sort} cmp $a->{$sort})
@@ -252,12 +300,8 @@ sub list : Chained('base') Args(0) {
     $response->{total} =
         eval {$rs->pager->total_entries} || scalar @{$response->{rows}};
 
-    # user sorted by FK so do the paging now (will be S-L-O-W)
-    if ($page =~ m/^\d+$/ and $limit =~ m/^\d+$/
-        and ($delayed_paging
-             or $info->{cols}->{$sort}->{is_fk}
-             or $info->{cols}->{$sort}->{is_rr})) {
-
+    # user filtered by FK so do the paging now (will be S-L-O-W)
+    if ($page =~ m/^\d+$/ and $limit =~ m/^\d+$/ and scalar keys %delay_page_sort) {
         my $pg = Data::Page->new;
         $pg->total_entries(scalar @{$response->{rows}});
         $pg->entries_per_page($limit);
@@ -286,9 +330,6 @@ sub list : Chained('base') Args(0) {
     }
     unshift @{$response->{rows}}, \%searchrow;
 
-    #$c->model($lf->{model})->result_source->storage->debug(0)
-    #    if $c->debug;
-
     return $self;
 }
 
@@ -444,8 +485,7 @@ sub _process_row_stack {
         my $info = $lf->{table_info}->{$model};
         foreach my $col (keys %{$info->{cols}}) {
             my $ci = $info->{cols}->{$col};
-            next unless exists $ci->{is_fk}
-                and exists $stashed_keys{$ci->{fk_model}};
+            next unless $ci->{is_fk} and exists $stashed_keys{$ci->{fk_model}};
             $col = $ci->{masked_col} if exists $ci->{masked_col};
             $data->{$col} = $stashed_keys{$ci->{fk_model}};
         }
@@ -497,12 +537,12 @@ sub list_stringified : Chained('base') Args(0) {
 
     # sanity check foreign key, and set up string part search
     $fk =~ s/\s//g; $fk =~ s/^[^.]*\.//;
-    $query = ($query ? qr/\Q$query\E/i : qr/./);
+    my $query_re = ($query ? qr/\Q$query\E/i : qr/./);
 
     if (!$fk
         or !exists $lf->{main}->{cols}->{$fk}
-        or not (exists $lf->{main}->{cols}->{$fk}->{is_fk}
-            or exists $lf->{main}->{cols}->{$fk}->{is_rr})) {
+        or not ($lf->{main}->{cols}->{$fk}->{is_fk}
+            or $lf->{main}->{cols}->{$fk}->{is_rr})) {
 
         $c->stash->{json_data} = {total => 0, rows => []};
         return $self;
@@ -510,10 +550,21 @@ sub list_stringified : Chained('base') Args(0) {
     
     my $rs = $c->model($lf->{model})
                 ->result_source->related_source($fk)->resultset;
+    my @data = ();
 
-    my @data =  map  { { dbid => $_->id, stringified => _sfy($_) } }
-                grep { _sfy($_) =~ m/$query/ } $rs->all;
-    @data = sort { $a->{stringified} cmp $b->{stringified} } @data;
+    # first try a simple and quick primary key search
+    if (my $single_result = $rs->find($query)) {
+        @data = ({
+            dbid => $single_result->id,
+            stringified => _sfy($single_result),
+        });
+    }
+    else {
+        # do the full text search
+        my @results =  map  { { dbid => $_->id, stringified => _sfy($_) } }
+                       grep { _sfy($_) =~ m/$query_re/ } $rs->all;
+        @data = sort { $a->{stringified} cmp $b->{stringified} } @results;
+    }
 
     my $pg = Data::Page->new;
     $pg->total_entries(scalar @data);
